----------------------------------------------------------------------------------
-- Company: 
-- Engineer: 
-- 
-- Create Date:    08:00:35 08/12/2010 
-- Design Name: 
-- Module Name:    Averaging - Behavioral 
-- Project Name: 
-- Target Devices: 
-- Tool versions: 
-- Description: 
--
-- Dependencies: 
--
-- Revision: 
-- Revision 0.01 - File Created
-- Additional Comments: 
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.ALL;
use IEEE.std_logic_ARITH.ALL;
use IEEE.std_logic_UNSIGNED.ALL;
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--use IEEE.NUMERIC_STD.ALL;

-- Uncomment the following library declaration if instantiating
-- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity Averaging is
    Port ( START_TRIGGER 							: in  STD_LOGIC;
           SWITCH_BLOCK_RAMS						: in	std_logic;
			  ADC_DATA				 					: in  STD_LOGIC_VECTOR (15 downto 0);
			  ADDER_BYPASS								: in 	std_logic;
			  --ADDER_DATA_FROM_RAM1_ENABLE 		: in 	std_logic;	
			  DATA_FROM_ADDER_TO_SPRAM				: out std_logic_vector (24 downto 0);
			  SPRAM_DATA								: in  std_logic_vector (24 downto 0);
			  SPRAM_ADDRESS_AVERAGING_CYCLE   	: out std_logic_vector (12 downto 0);
			  RAM_WEA_AVERAGING_CYCLE				: out std_logic_vector (0 downto 0);	
			  CLK_100MHZ 								: in  STD_LOGIC;
			  RESET										: in  STD_LOGIC);
end Averaging;

architecture Behavioral of Averaging is


component Adder
	port (
	a: IN std_logic_VECTOR(24 downto 0);
	b: IN std_logic_VECTOR(15 downto 0);
	clk: IN std_logic;
	ce: IN std_logic;
	bypass: IN std_logic;
	s: OUT std_logic_VECTOR(24 downto 0));
end component;



Signal COUNT_RAM_ADRESS_ENABLE	: std_logic;
Signal SAMPLE_DATA_ENABLE			: std_logic;

Signal RAM_WEA_ENABLE				: std_logic;


Signal SPRAM_READ_ADRESS			: std_logic_vector (12 downto 0);--:="0000000000000";

Signal INIT_SPRAM_ADDRES_ENABLE	: std_logic;


--Signal SPRAM_DATA_INPUT				: std_logic_vector (24 downto 0):="0000000000000000000000000";

--Signal SPRAM_DATA_IN1				: std_logic_vector (24 downto 0):="0000000000000000000000000";


--Signal SPRAM_DATA_IN2				: std_logic_vector (24 downto 0):="0000000000000000000000000";



Signal ADDER_CLK_ENABLE				: std_logic;
Signal ADDER_DATA_INB				: std_logic_vector (15 downto 0);
Signal ADDER_DATA						: std_logic_vector (24 downto 0);
Signal ADDER_DATA_OUT				: std_logic_vector (24 downto 0);
Signal WRITE_DATA_ENABLE	 		: std_logic;



type FSM_AVERAGING_DATA				is (TRACKING,SAMPLING,WRITE_DATA,COUNT_RAM_ADDRESS,WRITE_ENABLE,WAIT_STATE);
Signal CURRENT_STATE					: FSM_AVERAGING_DATA:= TRACKING;
Signal NEXT_STATE						: FSM_AVERAGING_DATA;
Signal JUMP_TO_FIRST_STATE 		: std_logic:='0';



begin

			
	DATA_FROM_ADDER_TO_SPRAM			<= (others => '0') when RESET = '1' else
													 ADDER_DATA;

	SPRAM_ADDRESS_AVERAGING_CYCLE		<= (others => '0') when RESET = '1' else
													 SPRAM_READ_ADRESS;
	
	Component_4 : Adder
		port map (
			a => SPRAM_DATA,
			b => ADDER_DATA_INB,
			clk => CLK_100MHZ,
			ce => ADDER_CLK_ENABLE,
			bypass => ADDER_BYPASS,
			s => ADDER_DATA_OUT	);
			

---------------------------------------------------------------------------------------------
--  DIESE STATEMACHINE IST FR DEN ZYKLUS SAMPLING -> ADDIEREN -> SPEICHERN ZUSTNDIG
--
---------------------------------------------------------------------------------------------
	
	
	SWITCH_STATE: Process (CLK_100MHZ)
	Begin
		if falling_edge (CLK_100MHZ) then
			CURRENT_STATE <= NEXT_STATE;
		end if;
	End Process SWITCH_STATE;


	FSMAVERAGING_DATA: Process (CURRENT_STATE,START_TRIGGER, JUMP_TO_FIRST_STATE)
	Begin
	
													INIT_SPRAM_ADDRES_ENABLE	<= '0';
													RAM_WEA_ENABLE					<= '0';
													SAMPLE_DATA_ENABLE 			<= '0';
													COUNT_RAM_ADRESS_ENABLE		<= '0';
													ADDER_CLK_ENABLE				<= '0';	
													WRITE_DATA_ENABLE	 			<= '0';	
											
	case CURRENT_STATE is
	
	when TRACKING 				=>				INIT_SPRAM_ADDRES_ENABLE	<= '1'; -- SPRAM Adresse wird auf 0 gesetzt
																							
													if START_TRIGGER = '1' then -- falling edge des Square Wave Signals
														NEXT_STATE <= SAMPLING;
													else
														NEXT_STATE <= TRACKING;
													end if;


	when SAMPLING 					=>			SAMPLE_DATA_ENABLE 		 <= '1';
													if JUMP_TO_FIRST_STATE = '1' then
														NEXT_STATE <= TRACKING;
													else
														NEXT_STATE <= WRITE_ENABLE;
													end if;
																							

	when WRITE_ENABLE				=>			RAM_WEA_ENABLE				 <= '1';
													ADDER_CLK_ENABLE			 <= '1';
													NEXT_STATE <= WRITE_DATA;
	
	when WRITE_DATA				=>			RAM_WEA_ENABLE		 		 <= '1';
													WRITE_DATA_ENABLE	 		 <= '1';	
													NEXT_STATE <= WAIT_STATE;
	
	
	when WAIT_STATE						=> NEXT_STATE <= COUNT_RAM_ADDRESS;
	
	
	when COUNT_RAM_ADDRESS		=>			COUNT_RAM_ADRESS_ENABLE	 <= '1';
													
													if JUMP_TO_FIRST_STATE = '1' then
														NEXT_STATE <= TRACKING;
													else
														NEXT_STATE <= SAMPLING;
													end if;
	
	end case;
	
	end process FSMAVERAGING_DATA;



	-- DIESER PROZESS BESTIMMT WENN DIE DATEN VOM ADC IN DEN ADDER GESCHRIEBEN WERDEN
	SAMPLING_ADC_DATA : Process (CLK_100MHZ, RESET)
	Begin
		if RESET = '1' then
			ADDER_DATA_INB <= (others => '0');
			else
		if rising_edge(CLK_100MHZ) then
			if SAMPLE_DATA_ENABLE = '1' then
				ADDER_DATA_INB <= ADC_DATA;
			else
				ADDER_DATA_INB <= ADDER_DATA_INB;
			end if;
		end if;
		end if;
	end Process SAMPLING_ADC_DATA;
			
	-- DIESER PROZESS BESTIMMT DIE DATEN DAS SCHREIBEN DER AUSGANGSDATEN DER ADDER DIE ZUM SPRAM GEFHRT WERDEN
	RAM_WRITING_ENABLE : Process (CLK_100MHZ, RESET)
	Begin
		if RESET = '1' then
			ADDER_DATA 		<= (others => '0');
			--ADDER_DATA_OUT <= (others => '0');
			else
		if rising_edge(CLK_100MHZ) then
			if WRITE_DATA_ENABLE = '1' then
				ADDER_DATA	<= ADDER_DATA_OUT;
			else
				ADDER_DATA	<= ADDER_DATA;
			end if;
		end if;
		end if;
	end Process RAM_WRITING_ENABLE;

	
	
	-- DIESER PROZESS BESTIMMT DEN ZYKLUS DER SPRAM ADRESSEN DIE HOCHGESCHALTEN WERDEN MSSEN WENN DIE DATEN GESCHRIEBEN WERDEN MSSEN; DIESES SIGNAL WIRD WEITER GELEITET
	RAM_ADDRESS_COUNING : Process (CLK_100MHZ,RESET)
	Begin
	
		if RESET = '1' then
			SPRAM_READ_ADRESS <=(others => '0');
			else
		if rising_edge(CLK_100MHZ) then
			if COUNT_RAM_ADRESS_ENABLE = '1' then
				if SPRAM_READ_ADRESS  >= "1110000011111" then -- "1110000011111"
					SPRAM_READ_ADRESS <= "0000000000000";
					JUMP_TO_FIRST_STATE <= '1';
				else
					JUMP_TO_FIRST_STATE <= '0';
					SPRAM_READ_ADRESS <= SPRAM_READ_ADRESS + 1;
				end if;
			elsif INIT_SPRAM_ADDRES_ENABLE = '1' then
				SPRAM_READ_ADRESS <= "0000000000000";
				JUMP_TO_FIRST_STATE <= '0';
			else
				JUMP_TO_FIRST_STATE <= '0';
				SPRAM_READ_ADRESS <= SPRAM_READ_ADRESS;
			end if;
		end if;
		end if;
	end Process RAM_ADDRESS_COUNING;
		
	-- DIESER PROZESS BESTIMMT DEN ZYKLUS DES WRITE ENABLES FR DIE JEWEILGEN SPRAMS; DIESES SIGNAL WIRD WEITER GELEITET
	DATA_WRITING_TO_RAM: Process (CLK_100MHZ, RESET)
	Begin
		if RESET = '1' then
		RAM_WEA_AVERAGING_CYCLE <= (others => '0');
		else
		if rising_edge(CLK_100MHZ) then
			if RAM_WEA_ENABLE = '1' then
				RAM_WEA_AVERAGING_CYCLE <= "1";
			else
				RAM_WEA_AVERAGING_CYCLE <= "0";
			end if;
		end if;
		end if;
	end Process DATA_WRITING_TO_RAM;

end Behavioral;