
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

---- Uncomment the following library declaration if instantiating
---- any Xilinx primitives in this code.
--library UNISIM;
--use UNISIM.VComponents.all;

entity UART_TX is
port( 
	-- Systemtakt
	CLK_10MHZ	: in	std_logic;
    
    -- Input
    data_in     : in    std_logic_vector (63 downto 0);
    data_en     : in    std_logic;
    
	-- Serielle Daten Ausgabe ber USBtoSerial Chip
    USB_1_CTS   : out 	std_logic;
    USB_1_RTS   : in 	std_logic;
    USB_1_RX    : out 	std_logic;
    USB_1_TX    : in 	std_logic;

    -- Debug Signale
    dbg         : out   std_logic;
    dbgv        : out   std_logic_vector (7 downto 0);
    
    -- Reset
	RESET		: in	std_logic
    );
end UART_TX;

architecture Behavioral of UART_TX is

component fifo_uart
port(
	wr_clk      : IN    std_logic;
	rd_clk      : IN    std_logic;
	rst         : IN    std_logic;
	din         : IN    std_logic_VECTOR(63 downto 0);
	wr_en       : IN    std_logic;
	rd_en       : IN    std_logic;
	dout        : OUT   std_logic_VECTOR(7 downto 0);
	full        : OUT   std_logic;
	empty       : OUT   std_logic
    );
end component;


type State is (STARTBIT,BIT_1,BIT_2,BIT_3,BIT_4,BIT_5,BIT_6,BIT_7,BIT_8,STOPBIT,SLEEP);
signal CurrentState, NextState  : State := SLEEP;						-- Zwischensignal fr STATE MACHINE 

signal cnt_baud     : std_logic_vector ( 6 downto 0 )   := "0000000"; 

signal bytetosend   : std_logic_vector (7 downto 0)     ; -- A
signal read_byte_en : std_logic;

signal no_data_to_send : std_logic;
signal CLK_115200   : std_logic;
begin

---------------------------------------------------------------------
-- Sende Fifo
---------------------------------------------------------------------
FIFO : fifo_uart
port map (
    wr_clk  => CLK_10MHZ,
    rd_clk  => CLK_115200,
    rst     => RESET,
    din     => data_in,
    wr_en   => data_en,
    rd_en   => read_byte_en,
    dout    => bytetosend,
    full    => open,
    empty   => no_data_to_send
    );   

USB_1_CTS <= '1';

dbgv <= bytetosend;
dbg <= no_data_to_send;

---------------------------------------------------------------------
--  State Maschine zum Raustakten der Daten
---------------------------------------------------------------------

uart_tx_fsm : process (CurrentState, bytetosend, no_data_to_send)
begin
    case CurrentState is
        when STARTBIT =>
            USB_1_RX <= '0';
            read_byte_en <= '1';
            NextState <= BIT_1;
        when BIT_1 =>
            USB_1_RX <= bytetosend(0);
            read_byte_en <= '0';
            NextState <= BIT_2;
        when BIT_2 =>
            USB_1_RX <= bytetosend(1);
            read_byte_en <= '0';
            NextState <= BIT_3;
        when BIT_3 =>
            USB_1_RX <= bytetosend(2);
            read_byte_en <= '0';
            NextState <= BIT_4;
        when BIT_4 =>
            USB_1_RX <= bytetosend(3);
            read_byte_en <= '0';
            NextState <= BIT_5;
        when BIT_5 =>
            USB_1_RX <= bytetosend(4);
            read_byte_en <= '0';
            NextState <= BIT_6;
        when BIT_6 =>
            USB_1_RX <= bytetosend(5);
            read_byte_en <= '0';
            NextState <= BIT_7;
        when BIT_7 =>
            USB_1_RX <= bytetosend(6);
            read_byte_en <= '0';
            NextState <= BIT_8;
        when BIT_8 =>
            USB_1_RX <= bytetosend(7);
            read_byte_en <= '0';
            NextState <= STOPBIT;
        when STOPBIT =>
            USB_1_RX <= '1';
            read_byte_en <= '0';
            NextState <= SLEEP;
        when SLEEP =>
            USB_1_RX <= '1';
            read_byte_en <= '0';
            if no_data_to_send = '0' then
                NextState <= STARTBIT;
            else
                NextState <= SLEEP;
            end if;
        when others => 
            USB_1_RX <= '1';
            read_byte_en <= '0';
            NextState <= SLEEP;
    end case;
end process;


---------------------------------------------------------------------
--  Timing of the State Machine
--  aus Baudrate erzeugen
---------------------------------------------------------------------
uart_fsm_seq : process (CLK_115200)
begin
    if falling_edge(CLK_115200) then
        CurrentState <= NextState;
    end if;
end process uart_fsm_seq; 


---------------------------------------------------------------------
--  Baudrate erzeugen
--  10.000.000 Hz Systemtakt => 115.200 HZ => 87 => 101 0111
---------------------------------------------------------------------
process (CLK_10MHZ) 
begin
    if falling_edge(CLK_10MHZ) then
        if cnt_baud = x"56" then    -- 87 - 1
            cnt_baud <= "0000000";
            CLK_115200 <= '0';
        elsif cnt_baud = x"2B" then
            cnt_baud <= cnt_baud + '1';
            CLK_115200 <= '1';
        else
            cnt_baud <= cnt_baud + '1';
        end if;
     end if;
end process;


end Behavioral;

